/*
 * Copyright (c) 2023 Xilinx
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this package; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#include <gst/video/gstvideoutils.h>

#define DYNAMIC_PARAMS_FILE "/tmp/dyn_params.txt"

static const gchar *dynamic_params_data =
    "10:NumB=3,BRkbps=5000000\n"
    "20:tAQ=1,sAQ=1,tAQGain=10,sAQGain=20\n"
    "30:MinQP=5,MaxQP=40\n";

static void
_on_frame_received (GstElement *encoder, GstVideoCodecFrame *frame,
    GstStructure *enc_params)
{
  if (frame->system_frame_number % 10 == 0) {
    /* Make every tenth frame a key frame. */
    GST_VIDEO_CODEC_FRAME_SET_FORCE_KEYFRAME (frame);
  }

  if (gst_structure_n_fields (enc_params) == 0) {
    /* Set encoding parameters unless they were read from the parameters file. */
    gst_structure_set (enc_params,
        "spatial-aq-gain", G_TYPE_UINT, 10,
        "temporal-aq-gain", G_TYPE_UINT, 20,
        "spatial-aq-mode", G_TYPE_BOOLEAN, TRUE,
        "temporal-aq-mode", G_TYPE_BOOLEAN, TRUE,
        "bitrate-kbps", G_TYPE_UINT, 10000,
        "num-b-frames", G_TYPE_UINT, 2,
        "min-qp", G_TYPE_UINT, 3,
        "max-qp", G_TYPE_UINT, 30,
        NULL);
  }
}

int
main (int argc, char **argv)
{
  GMainLoop *loop;
  GstElement *pipeline;
  GstElement *encoder;
  GstBus *bus;

  if (!g_file_set_contents (DYNAMIC_PARAMS_FILE, dynamic_params_data,
      strlen (dynamic_params_data), NULL)) {
    gst_println ("Couldn't write dynamic encoding parameters file.");
    return -1;
  }

  gst_init (&argc, &argv);

  pipeline = gst_parse_launch ("videotestsrc num-buffers=100 ! ama_upload ! "
      "ama_h264enc encoding-params-file=" DYNAMIC_PARAMS_FILE " name=enc ! "
      "fakesink", NULL);

  encoder = gst_bin_get_by_name (GST_BIN (pipeline), "enc");
  g_signal_connect (encoder, "frame-received", G_CALLBACK (_on_frame_received),
      NULL);
  gst_clear_object (&encoder);

  loop = g_main_loop_new (NULL, FALSE);

  bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
  gst_bus_add_signal_watch (bus);
  g_signal_connect_swapped (bus, "message::eos",
      G_CALLBACK (g_main_loop_quit), loop);
  gst_clear_object (&bus);

  gst_element_set_state (pipeline, GST_STATE_PLAYING);

  g_main_loop_run (loop);

  gst_println ("Exited the main loop");

  gst_element_set_state (pipeline, GST_STATE_NULL);

  gst_clear_object (&pipeline);
  g_clear_pointer(&loop, g_main_loop_unref);
}
